//*****************************************
// AVR ATmega328 Temperature Program
// CPU Clock:1MHz
//   iq[Yf[^@āAOIV[^(8MHz)ɂj
// ԉxƒlۑvO
//
// File Name:lcd.c
//
//  2011/12/17 Taishi New
//*****************************************

#include	<avr/pgmspace.h>
#include	<avr/io.h>

#include	"ATmega328_ONDO2.h"
#include	"timer.h"
#include	"lcd.h"

#ifdef	I2C_LCD
#else	//I2C_LCD
// **********************************************************
// ************************** LCD ***************************
// **********************************************************
//
//******************************
//****** CG RAM DATA 5x7 *******
//******************************
unsigned char	lcd_cg_data[64] PROGMEM  = {
										//   12345
										0b00000000, //1				// Code:0x00(Not Use)
										0b00000000, //2
										0b00000000, //3
										0b00000000, //4
										0b00000000, //5
										0b00000000, //6
										0b00000000, //7
										0b00000000,

										0b00000000,				// Code:0x01
										0b00000000,				// Battery Left Side
										0b00000111,
										0b00011100,
										0b00010000,
										0b00011100,
										0b00000111,
										0b00000000,

										0b00000000,				// Code:0x02
										0b00000000,				// Battery Right Side
										0b11111111,
										0b00000001,
										0b00000001,
										0b00000001,
										0b11111111,
										0b00000000,

										0b00000000,				// Code:0x03
										0b00010101,				// LED
										0b00001110,
										0b00011111,
										0b00010001,
										0b00010001,
										0b00011111,
										0b00000000,

										0b00000000,				// Code:0x04
										0b00000011,
										0b00001101,
										0b00001101,
										0b00001101,
										0b00000011,
										0b00000001,
										0b00000000,

										0b00000000,				// Code:0x05
										0b00000000,
										0b00000000,
										0b00000000,
										0b00000000,
										0b00000000,
										0b00000000,
										0b00000000,

										0b00000000,				// Code:0x06
										0b00000000,
										0b00000000,
										0b00000000,
										0b00000000,
										0b00000000,
										0b00000000,
										0b00000000,

										0b00000000,				// Code:0x07
										0b00000000,
										0b00000000,
										0b00000000,
										0b00000000,
										0b00000000,
										0b00000000,
										0b00000000,

									};


//******************************
// LCD Data(4bit) Output
//  uc : Output Data(D4 -- D7)
//  rs : RS bit Data(0=Command, 1=Data)
// PD6:RS
// PD4:E
// PC3:DB7
// PC2:DB6
// PC1:DB5
// PC0:DB4
// *****************************
void	lcd_out_4bit(unsigned char uc, int rs)
{
	if((rs & 0x01) == 0){			// RS:0(Command)
		PORTD &= ~(1 << PD6);
	}else{							// RS:1(Data)
		PORTD |=  (1 << PD6);
	}

	PORTD |=  (1 << PD4);			// E:1

	PORTC = ( (PORTC & ~((1 << PC0) | (1 << PC1) | (1 << PC2) | (1 << PC3))) | (uc >> 4) );	// Data


	PORTD &= ~(1 << PD4);			// E:0
}
//******************************
// LCD Command Output
//  uc : Command Data(D0 -- D7)
// *****************************
void	lcd_cmd_out(unsigned char uc)
{
	lcd_out_4bit(uc, 0);
	lcd_out_4bit((uc << 4), 0);
}
//******************************
// LCD Data Output
//  uc : Data(D0 -- D7)
// *****************************
void	lcd_data_out(unsigned char uc)
{
	lcd_out_4bit(uc, 1);
	lcd_out_4bit((uc << 4), 1);
}

//******************************
// LCD CG Data Set
// *****************************
void	lcd_cg_data_set(void)
{
	unsigned char	uc;
	unsigned char	i;

	lcd_cmd_out(0x40);							// CG RAM Address Set
#ifdef	MHz_1
	delay_16us(DELAY_LCD_us / 16);
#else	//MHz_1
	delay_2us(DELAY_LCD_us / 2);
#endif	//MHz_1

	for(i=0;i<64;i++){
		uc = pgm_read_byte(lcd_cg_data + i);
		lcd_data_out(uc);
#ifdef	MHz_1
		delay_16us(DELAY_LCD_us / 16);
#else	//MHz_1
		delay_2us(DELAY_LCD_us / 2);
#endif	//MHz_1
	}
}
//******************************
// Init. LCD
// *****************************
void	init_lcd(void)
{

	delay_ms(15);					// Wait 15mS
	lcd_out_4bit(0x30,0);
	delay_ms(5);					// Wait 5mS
	lcd_out_4bit(0x30,0);
	delay_ms(1);					// Wait 1mS
	lcd_out_4bit(0x30,0);

	delay_ms(1);					// Wait 1mS
	lcd_out_4bit(0x20,0);			// 4bit Mode

	delay_ms(1);					// Wait 1mS
	lcd_cmd_out(0x28);				// 4bit Mode 2line 5x7dot
#ifdef	MHz_1
	delay_16us(DELAY_LCD_us / 16);
#else	//MHz_1
	delay_2us(DELAY_LCD_us / 2);
#endif	//MHz_1
	lcd_cmd_out(0x08);				// Display OFF, Cursor OFF, Blink OFF
#ifdef	MHz_1
	delay_16us(DELAY_LCD_us / 16);
#else	//MHz_1
	delay_2us(DELAY_LCD_us / 2);
#endif	//MHz_1
	lcd_cmd_out(0x01);				// Display Clear
	delay_ms(2);
	lcd_cmd_out(0x06);				// Inclement, Corsor Move
#ifdef	MHz_1
	delay_16us(DELAY_LCD_us / 16);
#else	//MHz_1
	delay_2us(DELAY_LCD_us / 2);
#endif	//MHz_1
	lcd_cmd_out(0x02);				// Cursor Home
	delay_ms(2);
	lcd_cmd_out(0x0C);				// Display ON, Cursor OFF, Blink OFF
#ifdef	MHz_1
	delay_16us(DELAY_LCD_us / 16);
#else	//MHz_1
	delay_2us(DELAY_LCD_us / 2);
#endif	//MHz_1

	lcd_cg_data_set();				// LCD CG Data Set
}
//******************************
// Clear Display
// *****************************
void	disp_clear(void)
{
	lcd_cmd_out(0x01);				// Display Clear
	delay_ms(2);
	lcd_cmd_out(0x06);				// Inclement, Corsor Move
#ifdef	MHz_1
	delay_16us(DELAY_LCD_us / 16);
#else	//MHz_1
	delay_2us(DELAY_LCD_us / 2);
#endif	//MHz_1
	lcd_cmd_out(0x02);				// Cursor Home
	delay_ms(2);
	lcd_cmd_out(0x0C);				// Display ON, Cursor OFF, Blink OFF
#ifdef	MHz_1
	delay_16us(DELAY_LCD_us / 16);
#else	//MHz_1
	delay_2us(DELAY_LCD_us / 2);
#endif	//MHz_1

}

//******************************
// display LCD(16char x 2Line)
//  pos_x : Position X(0 -- 15)
//  pos_y : Position Y(0 -- 1)
//  ucp : String(termnate 0x00)
// *****************************
void	disp_lcd(unsigned char pos_x, unsigned char pos_y, unsigned char *ucp)
{
	unsigned char	pos;

	if((pos_x <= 15)&&(pos_y <= 1)){
		pos = pos_x;
		if(pos_y == 1){						// 2'nd Line
			pos += 0x40;
		}
		lcd_cmd_out(pos | 0x80);		// Set DDRAM ADR.
#ifdef	MHz_1
		delay_16us(DELAY_LCD_us / 16);
#else	//MHz_1
		delay_2us(DELAY_LCD_us / 2);
#endif	//MHz_1

		while(*ucp != 0x00){			// String Data Out
			lcd_data_out(*ucp);
			ucp++;
#ifdef	MHz_1
			delay_16us(DELAY_LCD_us / 16);
#else	//MHz_1
			delay_2us(DELAY_LCD_us / 2);
#endif	//MHz_1
		}
	}
}
#endif	//I2C_LCD
