//*****************************************
// AVR ATmega328 Temperature Program
// CPU Clock:1MHz
//   iq[Yf[^@WāAOIV[^(8MHz)ɂj
// ԉxƒlۑvO
//
// File Name:lcd_I2C.c
//
//  2012/08/10 Taishi New
//*****************************************

#include	<avr/pgmspace.h>
#include	<avr/io.h>
#include	<compat/twi.h>

#include	"ATmega328_ONDO2.h"

#ifdef	I2C_LCD

#include	"timer.h"
#include	"Mega_Twi.h"
#include	"port_bit.h"
#include	"uart.h"

#include	"lcd_I2C.h"


//static	unsigned char contrast = 0b101000;	// 3.0V lグƔZȂ܂B
unsigned char lcd_contrast = 0b111111;		// 2.4V lグƔZȂ܂BiꂪōZxj
static	unsigned char icon_dsp_buff[16];	// ACR̕ҏWp


// **********************************************************
// ************************** LCD ***************************
// **********************************************************
//
//
//******************************
// LCD POWER ON/OFF
//  on_off : 1=ON, 0=OFF
// *****************************
static
void	lcd_on_off(unsigned char on_off)
{
	if(on_off == 0){
		PORTB |=  (1 << PB1);		// LCD OFF
	}else{
		PORTB &= ~(1 << PB1);		// LCD ON
	}
}
//******************************
// LCD TWI Error
//  n : N of Blink
// |EEEE@uEvw
// *****************************
void	lcd_err_i2c(unsigned char n)
{
	unsigned char	i;

//	for(i = 0; i < n; i++){			// w񐔃uN
//		led_on_off(1);				// LED ON
//		delay_ms(100);				// 100mS Delay
//		led_on_off(0);				// LED OFF
//		delay_ms(400);				// 400mS Delay
//	}

	for(;;){						// ŉiv[v
		delay_ms(1000);				// 400mS Delay
		led_on_off(1);				// LED ON
		delay_ms(1000);				// 400mS Delay
		led_on_off(0);				// LED OFF
		delay_ms(1000);				// 400mS Delay
		for(i = 0; i < n; i++){			// w񐔃uN
			led_on_off(1);				// LED ON
			delay_ms(100);				// 100mS Delay
			led_on_off(0);				// LED OFF
			delay_ms(900);				// 400mS Delay
		}
	}
}

//******************************
// LCD Command Output
//  uc : Command Data(D0 -- D7)
// *****************************
static
void	lcd_cmd_out_i2c(unsigned char uc)
{
	unsigned char	r;

	r = twi_start(SB1602B_ID,SB1602B_ADDR,TW_WRITE);	// TWI Jn foCXhcƃfoCXAhXƂvqhsd
	if(r != 0){
		lcd_err_i2c(2);									// LCD I2C Error
	}
	r = twi_write(0x00);								// TWI RS=0 (Comand Select)
	if(r != 0){
		lcd_err_i2c(3);									// LCD I2C Error
	}
	r = twi_write(uc);									// TWI Comand Instruction
	if(r != 0){
		lcd_err_i2c(4);									// LCD I2C Error
	}
	twi_stop();

#ifdef	MHz_1
	delay_16us(DELAY_LCD_us / 16);
#else	//MHz_1
	delay_2us(DELAY_LCD_us / 2);
#endif	//MHz_1

}
//******************************
// LCD Data Output
//  uc : Data(D0 -- D7)
// *****************************
static
void	lcd_data_out_i2c(unsigned char uc)
{
	unsigned char	r;


	r = twi_start(SB1602B_ID,SB1602B_ADDR,TW_WRITE);	// TWI Jn foCXhcƃfoCXAhXƂvqhsd
	if(r != 0){
		lcd_err_i2c(5);									// LCD I2C Error
	}
	r = twi_write(0x40);								// TWI RS=1 (Data Select)
	if(r != 0){
		lcd_err_i2c(6);									// LCD I2C Error
	}
	r = twi_write(uc);									// TWI Data
	if(r != 0){
		lcd_err_i2c(7);									// LCD I2C Error
	}
	twi_stop();

#ifdef	MHz_1
	delay_16us(DELAY_LCD_us / 16);
#else	//MHz_1
	delay_2us(DELAY_LCD_us / 2);
#endif	//MHz_1
}

//******************************
// Init. LCD
// *****************************
void	init_lcd(void)
{
	lcd_on_off(1);					// LCD POWER ON

	delay_ms(100);					// 100mS Delay

	// I2C LCD̏s܂B
	lcd_cmd_out_i2c(0x38);			// Function Set 8bit 2Line 5x7dot

	lcd_cmd_out_i2c(0x39);			// Function Set 8bit 2Line 5x7dot IS=1(Instruction Table Access)

	lcd_cmd_out_i2c(0x14);			// Internal OSC frequency adj.

	lcd_cmd_out_i2c(0x70 | (lcd_contrast & 0xF));			// Contrast Set

	lcd_cmd_out_i2c(0x5c | ((lcd_contrast >> 4) & 0x3));	// Power/ICON/Contrast Set

	lcd_cmd_out_i2c(0x6c);			// Follower Control
	delay_ms(200);					// 300mS Delay

	lcd_cmd_out_i2c(0x38);			// Function Set 8bit 2Line 5x7dot

	lcd_cmd_out_i2c(0x08);			// Display OFF, Cursor OFF, Blink OFF

	lcd_cmd_out_i2c(0x01);			// Display Clear
	delay_ms(2);

	lcd_cmd_out_i2c(0x06);			// Inclement, Corsor Move

	lcd_cmd_out_i2c(0x02);			// Cursor Home
	delay_ms(2);

	lcd_cmd_out_i2c(0x0C);			// Display ON, Cursor OFF, Blink OFF

}
//******************************
// Close LCD
// *****************************
void	close_lcd(void)
{
	lcd_on_off(0);					// LCD POWER OFF
	delay_ms(100);					// 100mS Delay
}
//******************************
// Clear Display
// *****************************
void	disp_clear(void)
{
	lcd_cmd_out_i2c(0x01);			// Display Clear
	delay_ms(2);

	lcd_cmd_out_i2c(0x06);			// Inclement, Corsor Move

	lcd_cmd_out_i2c(0x02);			// Cursor Home
	delay_ms(2);

	lcd_cmd_out_i2c(0x0C);			// Display ON, Cursor OFF, Blink OFF
}
//******************************
// display LCD(16char x 2Line)
//  pos_x : Position X(0 -- 15)
//  pos_y : Position Y(0 -- 1)
//  ucp : String(termnate 0x00)
// *****************************
void	disp_lcd(unsigned char pos_x, unsigned char pos_y, unsigned char *ucp)
{
	unsigned char	pos;

	if((pos_x <= 15)&&(pos_y <= 1)){
		pos = pos_x;
		if(pos_y == 1){					// 2'nd Line
			pos += 0x40;
		}

		lcd_cmd_out_i2c(pos | 0x80);	// Set DDRAM ADR.

		while(*ucp != 0x00){			// String Data Out
			lcd_data_out_i2c(*ucp);
			ucp++;
		}
	}
}

//******************************
// Contrast Set
//******************************
void	lcd_contrast_set(void)
{
	lcd_cmd_out_i2c(0x70 | (lcd_contrast & 0xF));			// Contrast Set
	lcd_cmd_out_i2c(0x5c | ((lcd_contrast >> 4) & 0x3));	// Power/ICON/Contrast Set
}

// ************* LCD I2C TEST Start2 ***************


static unsigned char icon_data[]={
	// ACRAhX, Yrbg
	0x00, 0b10000,	// 0 Aei
	0x02, 0b10000,	// 1 db
	0x04, 0b10000,	// 2 uU[
	0x06, 0b10000,	// 3 Ђ`{
	
	0x07, 0b10000,	// 4 Op
	0x07, 0b01000,	// 5 Op
	0x09, 0b10000,	// 6 J
	0x0B, 0b10000,	// 7 oc
	
	0x0D, 0b01000,	// 8 obe@
	0x0D, 0b00100,	// 9 obe@
	0x0D, 0b00010,	//10 obe@O
	0x0D, 0b10000,	//11 obe@
	
	0x0F, 0b10000,	//12 ~c
};

// ̏icon_data[]̕тƓ
#define	N_ANTENA		0
#define	N_TELEPHON		1
#define	N_BUZZER		2
#define	N_HISIGATA		3
#define	N_UE_SANKAKU	4
#define	N_SITA_SANKAKU	5
#define	N_CAMERA		6
#define	N_BATU			7
#define	N_BAT_MID		8
#define	N_BAT_HIGH		9
#define	N_BATTERY		10
#define	N_BAT_LOW		11
#define	N_MITUWA		12


// ***************************************
// ACR̕\
//  
//     n:ACR̎w
//    on:0=\A1=\
// ***************************************
static
void	lcd_icon_select(unsigned char n, unsigned char on)
{
	if(on == 0){
		icon_dsp_buff[icon_data[n*2]] &= ~icon_data[n*2+1];	// obt@NA܂B
	}else{
		icon_dsp_buff[icon_data[n*2]] |= icon_data[n*2+1];	// obt@𗧂Ă܂B
	}

	lcd_cmd_out_i2c(0x39);									// R}h 0x39
	lcd_cmd_out_i2c(0x40+icon_data[n*2]);					// ACR̈̃AhXݒ 0x40+??
	lcd_data_out_i2c(icon_dsp_buff[icon_data[n*2]]);		// ACRf[^

}
// ***************************************
// Aei@ACR̕\
//  
//     on:0=\A1=\
// ***************************************
void	lcd_icon_antenna(unsigned char on)
{
	lcd_icon_select(N_ANTENA, on);			// ACR̕\
}
// ***************************************
// db@ACR̕\
//  
//     on:0=\A1=\
// ***************************************
void	lcd_icon_telephon(unsigned char on)
{
	lcd_icon_select(N_TELEPHON, on);			// ACR̕\
}
// ***************************************
// uU[@ACR̕\
//  
//     on:0=\A1=\
// ***************************************
void	lcd_icon_buzzer(unsigned char on)
{
	lcd_icon_select(N_BUZZER, on);			// ACR̕\
}
// ***************************************
// Ђ`@ACR̕\
//  
//     on:0=\A1=\
// ***************************************
void	lcd_icon_hisigata(unsigned char on)
{
	lcd_icon_select(N_HISIGATA, on);			// ACR̕\
}
// ***************************************
// Op@ACR̕\
//  
//     on:0=\A1=\
// ***************************************
void	lcd_icon_uesankaku(unsigned char on)
{
	lcd_icon_select(N_UE_SANKAKU, on);			// ACR̕\
}
// ***************************************
// Op@ACR̕\
//  
//     on:0=\A1=\
// ***************************************
void	lcd_icon_sitasankaku(unsigned char on)
{
	lcd_icon_select(N_SITA_SANKAKU, on);			// ACR̕\
}
// ***************************************
// J@ACR̕\
//  
//     on:0=\A1=\
// ***************************************
void	lcd_icon_camera(unsigned char on)
{
	lcd_icon_select(N_CAMERA, on);			// ACR̕\
}
// ***************************************
// oc@ACR̕\
//  
//     on:0=\A1=\
// ***************************************
void	lcd_icon_batu(unsigned char on)
{
	lcd_icon_select(N_BATU, on);			// ACR̕\
}
// ***************************************
// ~c@ACR̕\
//  
//     on:0=\A1=\
// ***************************************
void	lcd_icon_mituwa(unsigned char on)
{
	lcd_icon_select(N_MITUWA, on);			// ACR̕\
}
// ***************************************
// obe[t@ACR̕\
//   Ȃ
// ***************************************
void	lcd_icon_bat_full(void)
{
	icon_dsp_buff[icon_data[N_BAT_HIGH*2]] |= icon_data[N_BAT_HIGH*2+1];	// obt@𗧂Ă܂B
	icon_dsp_buff[icon_data[N_BAT_MID*2]] |= icon_data[N_BAT_MID*2+1];		// obt@𗧂Ă܂B
	icon_dsp_buff[icon_data[N_BAT_LOW*2]] |= icon_data[N_BAT_LOW*2+1];		// obt@𗧂Ă܂B
	icon_dsp_buff[icon_data[N_BATTERY*2]] |= icon_data[N_BATTERY*2+1];		// obt@𗧂Ă܂B
	lcd_cmd_out_i2c(0x39);													// R}h 0x39
	lcd_cmd_out_i2c(0x40+icon_data[N_BATTERY*2]);							// ACR̈̃AhXݒ 0x40+??
	lcd_data_out_i2c(icon_dsp_buff[icon_data[N_BATTERY*2]]);				// ACRf[^
}
// ***************************************
// obe[@ACR̕\
//   Ȃ
// ***************************************
void	lcd_icon_bat_mid(void)
{
	icon_dsp_buff[icon_data[N_BAT_HIGH*2]] &= ~icon_data[N_BAT_HIGH*2+1];	// obt@NA܂B
	icon_dsp_buff[icon_data[N_BAT_MID*2]] |= icon_data[N_BAT_MID*2+1];		// obt@𗧂Ă܂B
	icon_dsp_buff[icon_data[N_BAT_LOW*2]] |= icon_data[N_BAT_LOW*2+1];		// obt@𗧂Ă܂B
	icon_dsp_buff[icon_data[N_BATTERY*2]] |= icon_data[N_BATTERY*2+1];		// obt@𗧂Ă܂B
	lcd_cmd_out_i2c(0x39);													// R}h 0x39
	lcd_cmd_out_i2c(0x40+icon_data[N_BATTERY*2]);							// ACR̈̃AhXݒ 0x40+??
	lcd_data_out_i2c(icon_dsp_buff[icon_data[N_BATTERY*2]]);				// ACRf[^
}
// ***************************************
// obe[@ACR̕\
//   Ȃ
// ***************************************
void	lcd_icon_bat_low(void)
{
	icon_dsp_buff[icon_data[N_BAT_HIGH*2]] &= ~icon_data[N_BAT_HIGH*2+1];	// obt@NA܂B
	icon_dsp_buff[icon_data[N_BAT_MID*2]] &= ~icon_data[N_BAT_MID*2+1];		// obt@NA܂B
	icon_dsp_buff[icon_data[N_BAT_LOW*2]] |= icon_data[N_BAT_LOW*2+1];		// obt@𗧂Ă܂B
	icon_dsp_buff[icon_data[N_BATTERY*2]] |= icon_data[N_BATTERY*2+1];		// obt@𗧂Ă܂B
	lcd_cmd_out_i2c(0x39);													// R}h 0x39
	lcd_cmd_out_i2c(0x40+icon_data[N_BATTERY*2]);							// ACR̈̃AhXݒ 0x40+??
	lcd_data_out_i2c(icon_dsp_buff[icon_data[N_BATTERY*2]]);				// ACRf[^
}
// ***************************************
// obe[@ACR̕\
//   Ȃ
// ***************************************
void	lcd_icon_bat_empty(void)
{
	icon_dsp_buff[icon_data[N_BAT_HIGH*2]] &= ~icon_data[N_BAT_HIGH*2+1];	// obt@NA܂B
	icon_dsp_buff[icon_data[N_BAT_MID*2]] &= ~icon_data[N_BAT_MID*2+1];		// obt@NA܂B
	icon_dsp_buff[icon_data[N_BAT_LOW*2]] &= ~icon_data[N_BAT_LOW*2+1];		// obt@NA܂B
	icon_dsp_buff[icon_data[N_BATTERY*2]] |= icon_data[N_BATTERY*2+1];		// obt@𗧂Ă܂B
	lcd_cmd_out_i2c(0x39);													// R}h 0x39
	lcd_cmd_out_i2c(0x40+icon_data[N_BATTERY*2]);							// ACR̈̃AhXݒ 0x40+??
	lcd_data_out_i2c(icon_dsp_buff[icon_data[N_BATTERY*2]]);				// ACRf[^
}
// ***************************************
// obe[@ACR̔\
//   Ȃ
// ***************************************
void	lcd_icon_non_battery(void)
{
	icon_dsp_buff[icon_data[N_BAT_HIGH*2]] &= ~icon_data[N_BAT_HIGH*2+1];	// obt@NA܂B
	icon_dsp_buff[icon_data[N_BAT_MID*2]] &= ~icon_data[N_BAT_MID*2+1];		// obt@NA܂B
	icon_dsp_buff[icon_data[N_BAT_LOW*2]] &= ~icon_data[N_BAT_LOW*2+1];		// obt@NA܂B
	icon_dsp_buff[icon_data[N_BATTERY*2]] &= ~icon_data[N_BATTERY*2+1];		// obt@NA܂B
	lcd_cmd_out_i2c(0x39);													// R}h 0x39
	lcd_cmd_out_i2c(0x40+icon_data[N_BATTERY*2]);							// ACR̈̃AhXݒ 0x40+??
	lcd_data_out_i2c(icon_dsp_buff[icon_data[N_BATTERY*2]]);				// ACRf[^
}

#endif	//I2C_LCD
