//*****************************************
// AVR ATmega328 Temperature Program
// CPU Clock:1MHz
//   iq[Yf[^@āAOIV[^(8MHz)ɂj
// ԉxƒlۑvO
//
// File Name:timer.c
//
//  2011/12/17 Taishi New
//*****************************************

#include	<avr/interrupt.h>
#include	<avr/io.h>
#include	<avr/eeprom.h>

#include	"ATmega328_ONDO2.h"
#include	"gei_eeprom.h"
#include	"utl.h"
#include	"glb_dat.h"
#include	"port_bit.h"

#include	"timer.h"

#define	MHz_1	

static	unsigned char	tim_1ms;						// 10mSPʂJE^[

// **********************************************************
// ********************** Soft Timer ************************
// **********************************************************

volatile unsigned int	wait_i;
volatile unsigned int	wait_j;
volatile unsigned char	wait_us;

#ifdef	MHz_1
// ************************************
// Delay 16uS
// t: 16uS (max 255)
// ************************************
void delay_16us(unsigned char t)
{
	while(t-- != 0){
		wait_us++;				// 1uS Delay??
	}
}
//***************************
// Wait
//  tm : Wait Time(ms) max. 8000
// CPU Clock:1MHz
//***************************
void	delay_ms(int tm)
{

//	tm *= 2;
	for(wait_i = 0; wait_i < tm; wait_i++){
		delay_16us(63);
	}
}
#else	//MHz_1
// ************************************
// Delay uS
// t: uS (max 255)
// ************************************
void delay_2us(unsigned char t)
{
	while(t-- != 0){
		wait_us++;				// 1uS Delay??
		wait_us++;				// 1uS Delay??
	}
}
//***************************
// Wait
//  tm : Wait Time(ms) max. 8000
// CPU Clock:8MHz
//***************************
void	delay_ms(int tm)
{

	tm *= 2;
	for(wait_i = 0; wait_i < tm; wait_i++){
		delay_2us(250);
	}
}
#endif	//MHz_1

// **********************************************************
// ************************ Timer 2 *************************
// **********************************************************
//*****************************************
// Timer2 Initial
//  1mS 荞݂ɐݒ肷
//*****************************************
void	Timer2_init(void)
{
	PRR &= ~(1 << PRTIM2);								// Timer 1 ғ
#ifdef	MHz_1
	delay_16us(100/16);									// POOr҂
#else	//MHz_1
	delay_2us(100/2);									// POOr҂
#endif	//MHz_1

	TCCR2B = 0;											// Timer Stop
	TCNT2 = 0;											// Timer2 Count Set to ZERO
	TCCR2A = ((1 << WGM21) | (0 << WGM20));				// A Comper Match, Dont Output Timer2 Signal
	OCR2A  = COUNT_1MS;									// 1mS 8MHz/64=125kHz
#ifdef	MHz_1
	TCCR2B = ((0 << CS22) | (1 << CS21) | (0 << CS20));	// Timer Start (1MHz/8= 125kHz, 8uS)
#else	//MHz_1
	TCCR2B = ((1 << CS22) | (0 << CS21) | (0 << CS20));	// Timer Start (8MHz/64= 125kHz, 8uS)
#endif	//MHz_1
	TIMSK2 |= (1 << OCIE2A);							// A Comper Match Interrupt Enable

	tim_1ms = 0;										// 10mSPʂJE^[
	tim_10ms_led = 0;									// kdcpPOrPʃ^C}[
	tim_10ms_lcd = 0;									// kbcpPOrPʃ^C}[


}
//***************************
// Timer2 Comper Match A Interrupt
//
//  1mS 
//***************************
ISR	(TIMER2_COMPA_vect)
{

//	led_on_off(1);								// LED ON

	tim_1ms--;
	if(tim_1ms == 0){
		tim_1ms = 10;

		if(tim_10ms_led != 0){
			tim_10ms_led--;						// kdcpPOrPʃ^C}[@|P
		}

		if(tim_10ms_lcd != 0){
			tim_10ms_lcd--;						// kbcpPOrPʃ^C}[@|P
		}

		if(tim_10ms_sw != 0){
			tim_10ms_sw--;
		}

		if(tim_10ms_swr_continue != 0){
			tim_10ms_swr_continue--;			// EXCb`씻ʗpPOrPʃ^C}[
		}

		if(tim_10ms_swr_length != 0){
			tim_10ms_swr_length--;				// EXCb`씻ʗpPOrPʃ^C}[
		}

	}

//	led_on_off(0);								// LED OFF
}

